#ifndef FUNCTIONAL_HELPERS_HH
#define FUNCTIONAL_HELPERS_HH

// functional programming helper functions
// Copyright 2001 Anthony Williams

#include "functional_classes_msvc.hh"
#include "functional_refholder.hh"

namespace functional
{
    // classes to choose the template based on return type
    template<typename ReturnType>
    struct FR
    {
	template<typename Fun>
	struct F
	{
	    typedef FunctorAdaptor<ReturnType,Fun> FunctorAdaptor;
	    typedef MemberFunctionAdaptorObjectByPtr<ReturnType,Fun> MemberFunctionAdaptorObjectByPtr;
	    typedef MemberFunctionAdaptorObjectByRef<ReturnType,Fun> MemberFunctionAdaptorObjectByRef;
	};
	template<typename Fun,typename Arg>
	struct B
	{
	    typedef Binder1st<ReturnType,Fun,Arg> Binder1st;
	    typedef Binder2nd<ReturnType,Fun,Arg> Binder2nd;
	    typedef Binder3rd<ReturnType,Fun,Arg> Binder3rd;
	    typedef Binder4th<ReturnType,Fun,Arg> Binder4th;
	    typedef Binder5th<ReturnType,Fun,Arg> Binder5th;
	    typedef Binder6th<ReturnType,Fun,Arg> Binder6th;
	    typedef Binder7th<ReturnType,Fun,Arg> Binder7th;
	    typedef Binder8th<ReturnType,Fun,Arg> Binder8th;
	    typedef Binder9th<ReturnType,Fun,Arg> Binder9th;
	    typedef Binder10th<ReturnType,Fun,Arg> Binder10th;
	    typedef Binder11th<ReturnType,Fun,Arg> Binder11th;
	    typedef Binder12th<ReturnType,Fun,Arg> Binder12th;
	    typedef Binder13th<ReturnType,Fun,Arg> Binder13th;
	    typedef Binder14th<ReturnType,Fun,Arg> Binder14th;
	    typedef Binder15th<ReturnType,Fun,Arg> Binder15th;
	    typedef Binder16th<ReturnType,Fun,Arg> Binder16th;
	};
    };

    // specialize for "void" to get the RV versions.
    template<>
    struct FR<void>
    {
	template<typename Fun>
	struct F
	{
	    typedef FunctorAdaptorRV<Fun> FunctorAdaptor;
	    typedef MemberFunctionAdaptorObjectByPtrRV<Fun> MemberFunctionAdaptorObjectByPtr;
	    typedef MemberFunctionAdaptorObjectByRefRV<Fun> MemberFunctionAdaptorObjectByRef;
	};
	template<typename Fun,typename Arg>
	struct B
	{
	    typedef Binder1stRV<Fun,Arg> Binder1st;
	    typedef Binder2ndRV<Fun,Arg> Binder2nd;
	    typedef Binder3rdRV<Fun,Arg> Binder3rd;
	    typedef Binder4thRV<Fun,Arg> Binder4th;
	    typedef Binder5thRV<Fun,Arg> Binder5th;
	    typedef Binder6thRV<Fun,Arg> Binder6th;
	    typedef Binder7thRV<Fun,Arg> Binder7th;
	    typedef Binder8thRV<Fun,Arg> Binder8th;
	    typedef Binder9thRV<Fun,Arg> Binder9th;
	    typedef Binder10thRV<Fun,Arg> Binder10th;
	    typedef Binder11thRV<Fun,Arg> Binder11th;
	    typedef Binder12thRV<Fun,Arg> Binder12th;
	    typedef Binder13thRV<Fun,Arg> Binder13th;
	    typedef Binder14thRV<Fun,Arg> Binder14th;
	    typedef Binder15thRV<Fun,Arg> Binder15th;
	    typedef Binder16thRV<Fun,Arg> Binder16th;
	};
    };

    // helper functions - overloaded function "functor" will try and adapt anything
    template<typename FunctorType>
    FR<typename FunctorType::return_type>::F<
		   FunctorType>::FunctorAdaptor
    functor(FunctorType fun)
    {
	return fun;
    }
    template<typename ReturnType>
    FR<ReturnType>::F<
		   ReturnType (*)()>::FunctorAdaptor
    functor(ReturnType (*fun)())
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1)>::FunctorAdaptor
    functor(ReturnType (*fun)(Arg1))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2)>::FunctorAdaptor
    functor(ReturnType (*fun)(Arg1,Arg2))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3)>::FunctorAdaptor
    functor(ReturnType (*fun)(Arg1,Arg2,Arg3))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3,Arg4)>::FunctorAdaptor
    functor(ReturnType (*fun)(Arg1,Arg2,Arg3,Arg4))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3,Arg4,Arg5)>::FunctorAdaptor
    functor(ReturnType (*fun)(Arg1,Arg2,Arg3,Arg4,Arg5))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6)>::FunctorAdaptor
    functor(ReturnType (*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7)>::FunctorAdaptor
    functor(ReturnType (*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8)>::FunctorAdaptor
    functor(ReturnType (*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9)>::FunctorAdaptor
    functor(ReturnType (*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10)>::FunctorAdaptor
    functor(ReturnType (*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
				  Arg11)>::FunctorAdaptor
    functor(ReturnType (*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
			      Arg11))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
				  Arg11,Arg12)>::FunctorAdaptor
    functor(ReturnType (*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
			      Arg11,Arg12))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12,typename Arg13>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
				  Arg11,Arg12,Arg13)>::FunctorAdaptor
    functor(ReturnType (*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
			      Arg11,Arg12,Arg13))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12,typename Arg13,typename Arg14>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
				  Arg11,Arg12,Arg13,Arg14)>::FunctorAdaptor
    functor(ReturnType (*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
			      Arg11,Arg12,Arg13,Arg14))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12,typename Arg13,typename Arg14,typename Arg15>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
				  Arg11,Arg12,Arg13,Arg14,Arg15)>::FunctorAdaptor
    functor(ReturnType (*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
			      Arg11,Arg12,Arg13,Arg14,Arg15))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)()>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)())
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1)>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2)>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3)>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4)>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5)>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6)>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7)>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8)>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9)>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10)>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11)>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
					  Arg11))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11,Arg12)>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
					  Arg11,Arg12))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12,typename Arg13>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11,Arg12,Arg13)>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
					  Arg11,Arg12,Arg13))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12,typename Arg13,typename Arg14>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11,Arg12,Arg13,Arg14)>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
					  Arg11,Arg12,Arg13,Arg14))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12,typename Arg13,typename Arg14,typename Arg15>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11,Arg12,Arg13,Arg14,Arg15)>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
					  Arg11,Arg12,Arg13,Arg14,Arg15))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)() const>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)() const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1) const>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2) const>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3) const>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4) const>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5) const>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6) const>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7) const>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8) const>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9) const>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10) const>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11) const>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
					  Arg11) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11,Arg12) const>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
					  Arg11,Arg12) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12,typename Arg13>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11,Arg12,Arg13) const>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
					  Arg11,Arg12,Arg13) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12,typename Arg13,typename Arg14>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11,Arg12,Arg13,Arg14) const>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
					  Arg11,Arg12,Arg13,Arg14) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12,typename Arg13,typename Arg14,typename Arg15>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11,Arg12,Arg13,Arg14,Arg15) const>::MemberFunctionAdaptorObjectByPtr
    functor(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
					  Arg11,Arg12,Arg13,Arg14,Arg15) const)
    {
	return fun;
    }

    // helper functions that specify explicitly what we're adapting
    template<typename FunctorType>
    FR<typename FunctorType::return_type>::F<
		   FunctorType>::FunctorAdaptor
    genericFunctor(FunctorType fun)
    {
	return fun;
    }
    template<typename ReturnType>
    FR<ReturnType>::F<
		   ReturnType (*)()>::FunctorAdaptor
    pointerToFunction(ReturnType (*fun)())
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1)>::FunctorAdaptor
    pointerToFunction(ReturnType (*fun)(Arg1))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2)>::FunctorAdaptor
    pointerToFunction(ReturnType (*fun)(Arg1,Arg2))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3)>::FunctorAdaptor
    pointerToFunction(ReturnType (*fun)(Arg1,Arg2,Arg3))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3,Arg4)>::FunctorAdaptor
    pointerToFunction(ReturnType (*fun)(Arg1,Arg2,Arg3,Arg4))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3,Arg4,Arg5)>::FunctorAdaptor
    pointerToFunction(ReturnType (*fun)(Arg1,Arg2,Arg3,Arg4,Arg5))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6)>::FunctorAdaptor
    pointerToFunction(ReturnType (*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7)>::FunctorAdaptor
    pointerToFunction(ReturnType (*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8)>::FunctorAdaptor
    pointerToFunction(ReturnType (*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9)>::FunctorAdaptor
    pointerToFunction(ReturnType (*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10)>::FunctorAdaptor
    pointerToFunction(ReturnType (*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
				  Arg11)>::FunctorAdaptor
    pointerToFunction(ReturnType (*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
					Arg11))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
				  Arg11,Arg12)>::FunctorAdaptor
    pointerToFunction(ReturnType (*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
					Arg11,Arg12))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12,typename Arg13>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
				  Arg11,Arg12,Arg13)>::FunctorAdaptor
    pointerToFunction(ReturnType (*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
					Arg11,Arg12,Arg13))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12,typename Arg13,typename Arg14>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
				  Arg11,Arg12,Arg13,Arg14)>::FunctorAdaptor
    pointerToFunction(ReturnType (*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
					Arg11,Arg12,Arg13,Arg14))
    {
	return fun;
    }
    template<typename ReturnType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12,typename Arg13,typename Arg14,typename Arg15>
    FR<ReturnType>::F<
		   ReturnType (*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
				  Arg11,Arg12,Arg13,Arg14,Arg15)>::FunctorAdaptor
    pointerToFunction(ReturnType (*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
					Arg11,Arg12,Arg13,Arg14,Arg15))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)()>::MemberFunctionAdaptorObjectByPtr
    memberFunctionObjectByPtr(ReturnType (ObjectType::*fun)())
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1)>::MemberFunctionAdaptorObjectByPtr
    memberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2)>::MemberFunctionAdaptorObjectByPtr
    memberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3)>::MemberFunctionAdaptorObjectByPtr
    memberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4)>::MemberFunctionAdaptorObjectByPtr
    memberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5)>::MemberFunctionAdaptorObjectByPtr
    memberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6)>::MemberFunctionAdaptorObjectByPtr
    memberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7)>::MemberFunctionAdaptorObjectByPtr
    memberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8)>::MemberFunctionAdaptorObjectByPtr
    memberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9)>::MemberFunctionAdaptorObjectByPtr
    memberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10)>::MemberFunctionAdaptorObjectByPtr
    memberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11)>::MemberFunctionAdaptorObjectByPtr
    memberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
							    Arg11))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11,Arg12)>::MemberFunctionAdaptorObjectByPtr
    memberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
							    Arg11,Arg12))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12,typename Arg13>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11,Arg12,Arg13)>::MemberFunctionAdaptorObjectByPtr
    memberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
							    Arg11,Arg12,Arg13))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12,typename Arg13,typename Arg14>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11,Arg12,Arg13,Arg14)>::MemberFunctionAdaptorObjectByPtr
    memberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
							    Arg11,Arg12,Arg13,Arg14))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12,typename Arg13,typename Arg14,typename Arg15>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11,Arg12,Arg13,Arg14,Arg15)>::MemberFunctionAdaptorObjectByPtr
    memberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
							    Arg11,Arg12,Arg13,Arg14,Arg15))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)() const>::MemberFunctionAdaptorObjectByPtr
    constMemberFunctionObjectByPtr(ReturnType (ObjectType::*fun)() const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1) const>::MemberFunctionAdaptorObjectByPtr
    constMemberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2) const>::MemberFunctionAdaptorObjectByPtr
    constMemberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3) const>::MemberFunctionAdaptorObjectByPtr
    constMemberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4) const>::MemberFunctionAdaptorObjectByPtr
    constMemberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5) const>::MemberFunctionAdaptorObjectByPtr
    constMemberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6) const>::MemberFunctionAdaptorObjectByPtr
    constMemberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7) const>::MemberFunctionAdaptorObjectByPtr
    constMemberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8) const>::MemberFunctionAdaptorObjectByPtr
    constMemberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9) const>::MemberFunctionAdaptorObjectByPtr
    constMemberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10) const>::MemberFunctionAdaptorObjectByPtr
    constMemberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11) const>::MemberFunctionAdaptorObjectByPtr
    constMemberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								 Arg11) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11,Arg12) const>::MemberFunctionAdaptorObjectByPtr
    constMemberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								 Arg11,Arg12) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12,typename Arg13>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11,Arg12,Arg13) const>::MemberFunctionAdaptorObjectByPtr
    constMemberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								 Arg11,Arg12,Arg13) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12,typename Arg13,typename Arg14>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11,Arg12,Arg13,Arg14) const>::MemberFunctionAdaptorObjectByPtr
    constMemberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								 Arg11,Arg12,Arg13,Arg14) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12,typename Arg13,typename Arg14,typename Arg15>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11,Arg12,Arg13,Arg14,Arg15) const>::MemberFunctionAdaptorObjectByPtr
    constMemberFunctionObjectByPtr(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								 Arg11,Arg12,Arg13,Arg14,Arg15) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)()>::MemberFunctionAdaptorObjectByRef
    memberFunctionObjectByRef(ReturnType (ObjectType::*fun)())
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1)>::MemberFunctionAdaptorObjectByRef
    memberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2)>::MemberFunctionAdaptorObjectByRef
    memberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3)>::MemberFunctionAdaptorObjectByRef
    memberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4)>::MemberFunctionAdaptorObjectByRef
    memberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5)>::MemberFunctionAdaptorObjectByRef
    memberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6)>::MemberFunctionAdaptorObjectByRef
    memberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7)>::MemberFunctionAdaptorObjectByRef
    memberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8)>::MemberFunctionAdaptorObjectByRef
    memberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9)>::MemberFunctionAdaptorObjectByRef
    memberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10)>::MemberFunctionAdaptorObjectByRef
    memberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11)>::MemberFunctionAdaptorObjectByRef
    memberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
							    Arg11))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11,Arg12)>::MemberFunctionAdaptorObjectByRef
    memberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
							    Arg11,Arg12))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12,typename Arg13>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11,Arg12,Arg13)>::MemberFunctionAdaptorObjectByRef
    memberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
							    Arg11,Arg12,Arg13))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12,typename Arg13,typename Arg14>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11,Arg12,Arg13,Arg14)>::MemberFunctionAdaptorObjectByRef
    memberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
							    Arg11,Arg12,Arg13,Arg14))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12,typename Arg13,typename Arg14,typename Arg15>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11,Arg12,Arg13,Arg14,Arg15)>::MemberFunctionAdaptorObjectByRef
    memberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
							    Arg11,Arg12,Arg13,Arg14,Arg15))
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)() const>::MemberFunctionAdaptorObjectByRef
    constMemberFunctionObjectByRef(ReturnType (ObjectType::*fun)() const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1) const>::MemberFunctionAdaptorObjectByRef
    constMemberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2) const>::MemberFunctionAdaptorObjectByRef
    constMemberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3) const>::MemberFunctionAdaptorObjectByRef
    constMemberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4) const>::MemberFunctionAdaptorObjectByRef
    constMemberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5) const>::MemberFunctionAdaptorObjectByRef
    constMemberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6) const>::MemberFunctionAdaptorObjectByRef
    constMemberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7) const>::MemberFunctionAdaptorObjectByRef
    constMemberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8) const>::MemberFunctionAdaptorObjectByRef
    constMemberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9) const>::MemberFunctionAdaptorObjectByRef
    constMemberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10) const>::MemberFunctionAdaptorObjectByRef
    constMemberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11) const>::MemberFunctionAdaptorObjectByRef
    constMemberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								 Arg11) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11,Arg12) const>::MemberFunctionAdaptorObjectByRef
    constMemberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								 Arg11,Arg12) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12,typename Arg13>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11,Arg12,Arg13) const>::MemberFunctionAdaptorObjectByRef
    constMemberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								 Arg11,Arg12,Arg13) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12,typename Arg13,typename Arg14>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11,Arg12,Arg13,Arg14) const>::MemberFunctionAdaptorObjectByRef
    constMemberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								 Arg11,Arg12,Arg13,Arg14) const)
    {
	return fun;
    }
    template<typename ReturnType,typename ObjectType,
	     typename Arg1,typename Arg2,typename Arg3,typename Arg4,typename Arg5,
	     typename Arg6,typename Arg7,typename Arg8,typename Arg9,typename Arg10,
	     typename Arg11,typename Arg12,typename Arg13,typename Arg14,typename Arg15>
    FR<ReturnType>::F<
				     ReturnType (ObjectType::*)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								Arg11,Arg12,Arg13,Arg14,Arg15) const>::MemberFunctionAdaptorObjectByRef
    constMemberFunctionObjectByRef(ReturnType (ObjectType::*fun)(Arg1,Arg2,Arg3,Arg4,Arg5,Arg6,Arg7,Arg8,Arg9,Arg10,
								 Arg11,Arg12,Arg13,Arg14,Arg15) const)
    {
	return fun;
    }

    // helper functions for binders
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      Arg>::Binder1st
    bind1st(FunctorType fun,
	    Arg arg)
    {
	return Binder1st<FunctorType::return_type,
	    FunctorType,
	    Arg>(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      Arg>::Binder2nd
    bind2nd(FunctorType fun,
	    Arg arg)
    {
	return Binder2nd<FunctorType::return_type,
	    FunctorType,
	    Arg>(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      Arg>::Binder3rd
    bind3rd(FunctorType fun,
	    Arg arg)
    {
	return Binder3rd<FunctorType::return_type,
	    FunctorType,
	    Arg>(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      Arg>::Binder4th
    bind4th(FunctorType fun,
	    Arg arg)
    {
	return Binder4th<FunctorType::return_type,
	    FunctorType,
	    Arg>(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      Arg>::Binder5th
    bind5th(FunctorType fun,
	    Arg arg)
    {
	return Binder5th<FunctorType::return_type,
	    FunctorType,
	    Arg>(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      Arg>::Binder6th
    bind6th(FunctorType fun,
	    Arg arg)
    {
	return Binder6th<FunctorType::return_type,
	    FunctorType,
	    Arg>(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      Arg>::Binder7th
    bind7th(FunctorType fun,
	    Arg arg)
    {
	return Binder7th<FunctorType::return_type,
	    FunctorType,
	    Arg>(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      Arg>::Binder8th
    bind8th(FunctorType fun,
	    Arg arg)
    {
	return Binder8th<FunctorType::return_type,
	    FunctorType,
	    Arg>(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      Arg>::Binder9th
    bind9th(FunctorType fun,
	    Arg arg)
    {
	return Binder9th<FunctorType::return_type,
	    FunctorType,
	    Arg>(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      Arg>::Binder10th
    bind10th(FunctorType fun,
	    Arg arg)
    {
	return Binder10th<FunctorType::return_type,
	    FunctorType,
	    Arg>(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      Arg>::Binder11th
    bind11th(FunctorType fun,
	    Arg arg)
    {
	return Binder11th<FunctorType::return_type,
	    FunctorType,
	    Arg>(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      Arg>::Binder12th
    bind12th(FunctorType fun,
	    Arg arg)
    {
	return Binder12th<FunctorType::return_type,
	    FunctorType,
	    Arg>(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      Arg>::Binder13th
    bind13th(FunctorType fun,
	    Arg arg)
    {
	return Binder13th<FunctorType::return_type,
	    FunctorType,
	    Arg>(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      Arg>::Binder14th
    bind14th(FunctorType fun,
	    Arg arg)
    {
	return Binder14th<FunctorType::return_type,
	    FunctorType,
	    Arg>(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      Arg>::Binder15th
    bind15th(FunctorType fun,
	    Arg arg)
    {
	return Binder15th<FunctorType::return_type,
	    FunctorType,
	    Arg>(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      Arg>::Binder16th
    bind16th(FunctorType fun,
	    Arg arg)
    {
	return Binder16th<FunctorType::return_type,
	    FunctorType,
	    Arg>(fun,arg);
    }

    // bind arguments by reference
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      RefHolder<Arg> >::Binder1st
    bind1stByRef(FunctorType fun,
	    Arg& arg)
    {
	return Binder1st<FunctorType::return_type,
	    FunctorType,
	    RefHolder<Arg> >(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      RefHolder<Arg> >::Binder2nd
    bind2ndByRef(FunctorType fun,
	    Arg& arg)
    {
	return Binder2nd<FunctorType::return_type,
	    FunctorType,
	    RefHolder<Arg> >(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      RefHolder<Arg> >::Binder3rd
    bind3rdByRef(FunctorType fun,
	    Arg& arg)
    {
	return Binder3rd<FunctorType::return_type,
	    FunctorType,
	    RefHolder<Arg> >(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      RefHolder<Arg> >::Binder4th
    bind4thByRef(FunctorType fun,
	    Arg& arg)
    {
	return Binder4th<FunctorType::return_type,
	    FunctorType,
	    RefHolder<Arg> >(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      RefHolder<Arg> >::Binder5th
    bind5thByRef(FunctorType fun,
	    Arg& arg)
    {
	return Binder5th<FunctorType::return_type,
	    FunctorType,
	    RefHolder<Arg> >(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      RefHolder<Arg> >::Binder6th
    bind6thByRef(FunctorType fun,
	    Arg& arg)
    {
	return Binder6th<FunctorType::return_type,
	    FunctorType,
	    RefHolder<Arg> >(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      RefHolder<Arg> >::Binder7th
    bind7thByRef(FunctorType fun,
	    Arg& arg)
    {
	return Binder7th<FunctorType::return_type,
	    FunctorType,
	    RefHolder<Arg> >(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      RefHolder<Arg> >::Binder8th
    bind8thByRef(FunctorType fun,
	    Arg& arg)
    {
	return Binder8th<FunctorType::return_type,
	    FunctorType,
	    RefHolder<Arg> >(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      RefHolder<Arg> >::Binder9th
    bind9thByRef(FunctorType fun,
	    Arg& arg)
    {
	return Binder9th<FunctorType::return_type,
	    FunctorType,
	    RefHolder<Arg> >(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      RefHolder<Arg> >::Binder10th
    bind10thByRef(FunctorType fun,
	    Arg& arg)
    {
	return Binder10th<FunctorType::return_type,
	    FunctorType,
	    RefHolder<Arg> >(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      RefHolder<Arg> >::Binder11th
    bind11thByRef(FunctorType fun,
	    Arg& arg)
    {
	return Binder11th<FunctorType::return_type,
	    FunctorType,
	    RefHolder<Arg> >(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      RefHolder<Arg> >::Binder12th
    bind12thByRef(FunctorType fun,
	    Arg& arg)
    {
	return Binder12th<FunctorType::return_type,
	    FunctorType,
	    RefHolder<Arg> >(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      RefHolder<Arg> >::Binder13th
    bind13thByRef(FunctorType fun,
	    Arg& arg)
    {
	return Binder13th<FunctorType::return_type,
	    FunctorType,
	    RefHolder<Arg> >(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      RefHolder<Arg> >::Binder14th
    bind14thByRef(FunctorType fun,
	    Arg& arg)
    {
	return Binder14th<FunctorType::return_type,
	    FunctorType,
	    RefHolder<Arg> >(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      RefHolder<Arg> >::Binder15th
    bind15thByRef(FunctorType fun,
	    Arg& arg)
    {
	return Binder15th<FunctorType::return_type,
	    FunctorType,
	    RefHolder<Arg> >(fun,arg);
    }
    template<typename FunctorType,
	     typename Arg>
    FR<typename FunctorType::return_type>::B<
	      FunctorType,
	      RefHolder<Arg> >::Binder16th
    bind16thByRef(FunctorType fun,
	    Arg& arg)
    {
	return Binder16th<FunctorType::return_type,
	    FunctorType,
	    RefHolder<Arg> >(fun,arg);
    }
    

}

#endif
